@@ -19,30 +19,25 @@ package gitlab
 import (
 	"context"
 	"crypto/subtle"
-	"fmt"
-
-	"google.golang.org/grpc/codes"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
+	triggersv1 "github.com/tektoncd/triggers/pkg/apis/triggers/v1beta1"
 	"github.com/tektoncd/triggers/pkg/interceptors"
-
-	triggersv1 "github.com/tektoncd/triggers/pkg/apis/triggers/v1alpha1"
-
 	"go.uber.org/zap"
-	"k8s.io/client-go/kubernetes"
+	"google.golang.org/grpc/codes"
+	corev1lister "k8s.io/client-go/listers/core/v1"
 )
 
 var _ triggersv1.InterceptorInterface = (*Interceptor)(nil)
 
 type Interceptor struct {
-	KubeClientSet kubernetes.Interface
-	Logger        *zap.SugaredLogger
+	SecretLister corev1lister.SecretLister
+	Logger       *zap.SugaredLogger
 }
 
-func NewInterceptor(k kubernetes.Interface, l *zap.SugaredLogger) *Interceptor {
+func NewInterceptor(sl corev1lister.SecretLister, l *zap.SugaredLogger) *Interceptor {
 	return &Interceptor{
-		Logger:        l,
-		KubeClientSet: k,
+		SecretLister: sl,
+		Logger:       l,
 	}
 }
 
@@ -53,28 +48,8 @@ func (w *Interceptor) Process(ctx context.Context, r *triggersv1.InterceptorRequ
 	}
 
 	headers := interceptors.Canonical(r.Header)
-	if p.SecretRef != nil {
-		// Check the secret to see if it is empty
-		if p.SecretRef.SecretKey == "" {
-			return interceptors.Fail(codes.FailedPrecondition, "gitlab interceptor secretRef.secretKey is empty")
-		}
-		header := headers.Get("X-GitLab-Token")
-		if header == "" {
-			return interceptors.Fail(codes.InvalidArgument, "no X-GitLab-Token header set")
-		}
 
-		ns, _ := triggersv1.ParseTriggerID(r.Context.TriggerID)
-		secret, err := w.KubeClientSet.CoreV1().Secrets(ns).Get(ctx, p.SecretRef.SecretName, metav1.GetOptions{})
-		if err != nil {
-			return interceptors.Fail(codes.Internal, fmt.Sprintf("error getting secret: %v", err))
-		}
-		secretToken := secret.Data[p.SecretRef.SecretKey]
-
-		// Make sure to use a constant time comparison here.
-		if subtle.ConstantTimeCompare([]byte(header), secretToken) == 0 {
-			return interceptors.Fail(codes.InvalidArgument, "Invalid X-GitLab-Token")
-		}
-	}
+	// Check if the event type is in the allow-list
 	if p.EventTypes != nil {
 		actualEvent := headers.Get("X-GitLab-Event")
 		isAllowed := false
@@ -88,6 +63,29 @@ func (w *Interceptor) Process(ctx context.Context, r *triggersv1.InterceptorRequ
 			return interceptors.Failf(codes.FailedPrecondition, "event type %s is not allowed", actualEvent)
 		}
 	}
+
+	// Next validate secrets
+	if p.SecretRef != nil {
+		// Check the secret to see if it is empty
+		if p.SecretRef.SecretKey == "" {
+			return interceptors.Fail(codes.FailedPrecondition, "gitlab interceptor secretRef.secretKey is empty")
+		}
+		header := headers.Get("X-GitLab-Token")
+		if header == "" {
+			return interceptors.Fail(codes.InvalidArgument, "no X-GitLab-Token header set")
+		}
+
+		ns, _ := triggersv1.ParseTriggerID(r.Context.TriggerID)
+		secretToken, err := interceptors.GetSecretToken(nil, w.SecretLister, p.SecretRef, ns)
+		if err != nil {
+			return interceptors.Failf(codes.FailedPrecondition, "error getting secret: %v", err)
+		}
+
+		// Make sure to use a constant time comparison here.
+		if subtle.ConstantTimeCompare([]byte(header), secretToken) == 0 {
+			return interceptors.Fail(codes.InvalidArgument, "Invalid X-GitLab-Token")
+		}
+	}
 	return &triggersv1.InterceptorResponse{
 		Continue: true,
 	}
