@@ -24,22 +24,30 @@ import (
 	"fmt"
 	"io/ioutil"
 	"net/http"
+	"os"
+	"sync"
 
-	triggersv1 "github.com/tektoncd/triggers/pkg/apis/triggers/v1alpha1"
+	"github.com/tektoncd/triggers/pkg/apis/triggers"
+	triggersv1 "github.com/tektoncd/triggers/pkg/apis/triggers/v1beta1"
 	triggersclientset "github.com/tektoncd/triggers/pkg/client/clientset/versioned"
-	listers "github.com/tektoncd/triggers/pkg/client/listers/triggers/v1alpha1"
+	listersv1alpha1 "github.com/tektoncd/triggers/pkg/client/listers/triggers/v1alpha1"
+	listers "github.com/tektoncd/triggers/pkg/client/listers/triggers/v1beta1"
 	"github.com/tektoncd/triggers/pkg/interceptors"
 	"github.com/tektoncd/triggers/pkg/interceptors/webhook"
+	"github.com/tektoncd/triggers/pkg/reconciler/events"
 	"github.com/tektoncd/triggers/pkg/resources"
+	"github.com/tektoncd/triggers/pkg/sink/cloudevent"
 	"github.com/tektoncd/triggers/pkg/template"
 	"github.com/tidwall/sjson"
 	"go.uber.org/zap"
-	kerrors "k8s.io/apimachinery/pkg/api/errors"
+	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/labels"
 	discoveryclient "k8s.io/client-go/discovery"
 	"k8s.io/client-go/dynamic"
 	"k8s.io/client-go/kubernetes"
+	"k8s.io/client-go/tools/record"
+	v1 "knative.dev/pkg/apis/duck/v1"
 )
 
 // Sink defines the sink resource for processing incoming events for the
@@ -50,11 +58,18 @@ type Sink struct {
 	DiscoveryClient        discoveryclient.ServerResourcesInterface
 	DynamicClient          dynamic.Interface
 	HTTPClient             *http.Client
+	CEClient               cloudevent.CEClient
 	EventListenerName      string
 	EventListenerNamespace string
 	Logger                 *zap.SugaredLogger
 	Recorder               *Recorder
 	Auth                   AuthOverride
+	PayloadValidation      bool
+	CloudEventURI          string
+	// WGProcessTriggers keeps track of triggers or triggerGroups currently being processed
+	// Currently only used in tests to wait for all triggers to finish processing
+	WGProcessTriggers *sync.WaitGroup
+	EventRecorder     record.EventRecorder
 
 	// listers index properties about resources
 	EventListenerLister         listers.EventListenerLister
@@ -62,7 +77,7 @@ type Sink struct {
 	TriggerBindingLister        listers.TriggerBindingLister
 	ClusterTriggerBindingLister listers.ClusterTriggerBindingLister
 	TriggerTemplateLister       listers.TriggerTemplateLister
-	ClusterInterceptorLister    listers.ClusterInterceptorLister
+	ClusterInterceptorLister    listersv1alpha1.ClusterInterceptorLister
 }
 
 // Response defines the HTTP body that the Sink responds to events with.
@@ -81,120 +96,119 @@ type Response struct {
 	ErrorMessage string `json:"errorMessage,omitempty"`
 }
 
+func (r Sink) emitEvents(recorder record.EventRecorder, el *triggersv1.EventListener, eventType string, err error) {
+	if os.Getenv("EL_EVENT") == "enable" {
+		events.Emit(recorder, eventType, el, err)
+	}
+}
+
 // HandleEvent processes an incoming HTTP event for the event listener.
 func (r Sink) HandleEvent(response http.ResponseWriter, request *http.Request) {
 	log := r.Logger.With(
 		zap.String("eventlistener", r.EventListenerName),
 		zap.String("namespace", r.EventListenerNamespace),
 	)
-	el, err := r.EventListenerLister.EventListeners(r.EventListenerNamespace).Get(r.EventListenerName)
-	if err != nil {
-		log.Errorf("Error getting EventListener %s in Namespace %s: %s", r.EventListenerName, r.EventListenerNamespace, err)
-		response.WriteHeader(http.StatusInternalServerError)
-		return
+	eventID := template.UUID()
+	log = log.With(zap.String(triggers.EventIDLabelKey, eventID))
+
+	elTemp := triggersv1.EventListener{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "EventListener",
+			APIVersion: "v1beta1",
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      r.EventListenerName,
+			Namespace: r.EventListenerNamespace,
+		},
+		Spec: triggersv1.EventListenerSpec{
+			Resources: triggersv1.Resources{
+				KubernetesResource: &triggersv1.KubernetesResource{
+					WithPodSpec: v1.WithPodSpec{
+						Template: v1.PodSpecable{
+							Spec: corev1.PodSpec{
+								Containers: []corev1.Container{{
+									// enabled by default for temporary EL
+									Env: []corev1.EnvVar{{
+										Name:  "EL_EVENT",
+										Value: "true",
+									}},
+								}},
+							},
+						},
+					},
+				},
+			},
+		},
 	}
-	elUID := string(el.GetUID())
-	log = log.With(zap.String("eventlistenerUID", elUID))
+
+	r.emitEvents(r.EventRecorder, &elTemp, events.TriggerProcessingStartedV1, nil)
+	r.sendCloudEvents(request.Header, elTemp, eventID, events.TriggerProcessingStartedV1)
+
 	event, err := ioutil.ReadAll(request.Body)
 	if err != nil {
 		log.Errorf("Error reading event body: %s", err)
+		r.recordCountMetrics(failTag)
 		response.WriteHeader(http.StatusInternalServerError)
+		r.emitEvents(r.EventRecorder, &elTemp, events.TriggerProcessingFailedV1, err)
+		r.sendCloudEvents(request.Header, elTemp, eventID, events.TriggerProcessingFailedV1)
 		return
 	}
 
-	eventID := template.UUID()
-	log = log.With(zap.String(triggersv1.EventIDLabelKey, eventID))
-	log.Debugf("handling event with path %s, payload: %s and header: %v", request.URL.Path, string(event), request.Header)
-	var trItems []*triggersv1.Trigger
-	labelSelector := labels.Everything()
-	if el.Spec.LabelSelector != nil {
-		labelSelector, err = metav1.LabelSelectorAsSelector(el.Spec.LabelSelector)
-		if err != nil {
-			r.Logger.Errorf("Failed to create label selector: %s", err)
-			response.WriteHeader(http.StatusInternalServerError)
-			return
-		}
-	}
-	var triggerFunc func() ([]*triggersv1.Trigger, error)
-	switch {
-	case len(el.Spec.NamespaceSelector.MatchNames) == 1 && el.Spec.NamespaceSelector.MatchNames[0] == "*":
-		triggerFunc = func() ([]*triggersv1.Trigger, error) {
-			return r.TriggerLister.List(labelSelector)
-		}
-	case len(el.Spec.NamespaceSelector.MatchNames) != 0:
-		triggerFunc = func() ([]*triggersv1.Trigger, error) {
-			var trList []*triggersv1.Trigger
-			for _, v := range el.Spec.NamespaceSelector.MatchNames {
-				trNsList, err := r.TriggerLister.Triggers(v).List(labelSelector)
-				if err != nil {
-					return nil, err
-				}
-				trList = append(trList, trNsList...)
-			}
-			return trList, nil
-		}
-	case len(el.Spec.NamespaceSelector.MatchNames) == 0:
-		if el.Spec.LabelSelector != nil {
-			triggerFunc = func() ([]*triggersv1.Trigger, error) {
-				return r.TriggerLister.Triggers(el.GetNamespace()).List(labelSelector)
-			}
-		}
-	}
-	if triggerFunc != nil {
-		trList, err := triggerFunc()
-		if err != nil {
-			log.Errorf("Error getting Triggers: %s", err)
-			response.WriteHeader(http.StatusInternalServerError)
-			return
-		}
-		trItems = append(trItems, trList...)
-	}
-
-	triggers, err := r.merge(el.Spec.Triggers, trItems)
+	el, err := r.EventListenerLister.EventListeners(r.EventListenerNamespace).Get(r.EventListenerName)
 	if err != nil {
-		log.Errorf("Error merging Triggers: %s", err)
+		log.Errorf("Error getting EventListener %s in Namespace %s: %s", r.EventListenerName, r.EventListenerNamespace, err)
+		r.recordCountMetrics(failTag)
 		response.WriteHeader(http.StatusInternalServerError)
+		r.emitEvents(r.EventRecorder, &elTemp, events.TriggerProcessingFailedV1, err)
+		r.sendCloudEvents(request.Header, elTemp, eventID, events.TriggerProcessingFailedV1)
 		return
 	}
-	result := make(chan int, 10)
-	// Execute each Trigger
-	for _, t := range triggers {
+
+	elUID := string(el.GetUID())
+	log = log.With(zap.String("eventlistenerUID", elUID))
+
+	log = log.With(zap.String(triggers.EventIDLabelKey, eventID))
+	log.Debugf("handling event with path %s, payload: %s and header: %v", request.URL.Path, string(event), request.Header)
+	trItems, err := r.selectTriggers(el.Spec.NamespaceSelector, el.Spec.LabelSelector)
+	if err != nil {
+		r.Logger.Errorf("unable to select configured mergedTriggers: %s", err)
+		response.WriteHeader(http.StatusInternalServerError)
+		r.emitEvents(r.EventRecorder, el, events.TriggerProcessingFailedV1, err)
+		r.sendCloudEvents(nil, *el, eventID, events.TriggerProcessingFailedV1)
+		return
+	}
+
+	// Process any ungroupedTriggers
+	mergedTriggers, err := r.merge(el.Spec.Triggers, trItems)
+	if err != nil {
+		log.Errorf("error merging triggers: %s", err)
+		response.WriteHeader(http.StatusInternalServerError)
+		r.emitEvents(r.EventRecorder, el, events.TriggerProcessingFailedV1, err)
+		r.sendCloudEvents(nil, *el, eventID, events.TriggerProcessingFailedV1)
+		return
+	}
+	r.WGProcessTriggers.Add(len(mergedTriggers))
+	for _, t := range mergedTriggers {
 		go func(t triggersv1.Trigger) {
+			defer r.WGProcessTriggers.Done()
 			localRequest := request.Clone(request.Context())
-			if err := r.processTrigger(t, localRequest, event, eventID, log); err != nil {
-				if kerrors.IsUnauthorized(err) {
-					result <- http.StatusUnauthorized
-					return
-				}
-				if kerrors.IsForbidden(err) {
-					result <- http.StatusForbidden
-					return
-				}
-				result <- http.StatusAccepted
-				return
-			}
-			result <- http.StatusCreated
+			emptyExtensions := make(map[string]interface{})
+			r.processTrigger(t, el, localRequest, event, eventID, log, emptyExtensions)
 		}(*t)
 	}
 
-	// The eventlistener waits until all the trigger executions (up-to the creation of the resources) and
-	// only when at least one of the execution completed successfully, it returns response code 201(Created) otherwise it returns 202 (Accepted).
-	code := http.StatusAccepted
-	for i := 0; i < len(triggers); i++ {
-		thiscode := <-result
-		// current take - if someone is doing unauthorized stuff, we abort immediately;
-		// unauthorized should be the final status code vs. the less than comparison
-		// below around accepted vs. created
-		if thiscode == http.StatusUnauthorized || thiscode == http.StatusForbidden {
-			code = thiscode
-			break
-		}
-		if thiscode < code {
-			code = thiscode
-		}
+	// Process grouped triggers
+	for _, group := range el.Spec.TriggerGroups {
+		r.WGProcessTriggers.Add(1)
+		go func(g triggersv1.EventListenerTriggerGroup) {
+			defer r.WGProcessTriggers.Done()
+			localRequest := request.Clone(request.Context())
+			r.processTriggerGroups(g, el, localRequest, event, eventID, log, r.WGProcessTriggers)
+		}(group)
 	}
 
-	response.WriteHeader(code)
+	r.recordCountMetrics(successTag)
+	response.WriteHeader(http.StatusAccepted)
 	response.Header().Set("Content-Type", "application/json")
 	body := Response{
 		EventListener:    r.EventListenerName,
@@ -204,7 +218,36 @@ func (r Sink) HandleEvent(response http.ResponseWriter, request *http.Request) {
 	}
 	if err := json.NewEncoder(response).Encode(body); err != nil {
 		log.Errorf("failed to write back sink response: %v", err)
+		r.emitEvents(r.EventRecorder, el, events.TriggerProcessingFailedV1, err)
+		r.sendCloudEvents(nil, *el, eventID, events.TriggerProcessingFailedV1)
 	}
+	r.emitEvents(r.EventRecorder, el, events.TriggerProcessingDoneV1, nil)
+	r.sendCloudEvents(nil, *el, eventID, events.TriggerProcessingDoneV1)
+}
+
+func (r Sink) sendCloudEvents(headers http.Header, el triggersv1.EventListener, eventID, eventType string) {
+	data, err := json.Marshal(headers)
+	if err != nil {
+		r.Logger.Errorf("Error marshaling request Headers to json: %s", err)
+		return
+	}
+
+	// If no cloudEventURI, then don't try to sendCloudEvents
+	if r.CloudEventURI == "" {
+		return
+	}
+
+	resource := cloudevent.Resource{
+		EventID:   eventID,
+		EventType: eventType,
+		TargetURI: r.CloudEventURI,
+		Client:    r.CEClient,
+		Logger:    r.Logger,
+		Data:      data,
+		EL:        el,
+	}
+
+	go resource.SendCloudEvents()
 }
 
 func (r Sink) merge(et []triggersv1.EventListenerTrigger, trItems []*triggersv1.Trigger) ([]*triggersv1.Trigger, error) {
@@ -237,19 +280,112 @@ func (r Sink) merge(et []triggersv1.EventListenerTrigger, trItems []*triggersv1.
 	return triggers, nil
 }
 
-func (r Sink) processTrigger(t triggersv1.Trigger, request *http.Request, event []byte, eventID string, eventLog *zap.SugaredLogger) error {
-	log := eventLog.With(zap.String(triggersv1.TriggerLabelKey, t.Name))
+func (r Sink) processTriggerGroups(g triggersv1.EventListenerTriggerGroup, el *triggersv1.EventListener, request *http.Request, event []byte, eventID string, eventLog *zap.SugaredLogger, wg *sync.WaitGroup) {
+	log := eventLog.With(zap.String(triggers.TriggerGroupLabelKey, g.Name))
 
-	finalPayload, header, iresp, err := r.ExecuteInterceptors(t, request, event, log, eventID)
+	extensions := map[string]interface{}{}
+	payload, header, resp, err := r.ExecuteInterceptors(g.Interceptors, request, event, log, eventID, fmt.Sprintf("namespaces/%s/triggerGroups/%s", r.EventListenerNamespace, g.Name), r.EventListenerNamespace, extensions)
 	if err != nil {
 		log.Error(err)
-		return err
+		return
+	}
+	if resp != nil {
+		if resp.Extensions != nil {
+			for k, v := range resp.Extensions {
+				extensions[k] = v
+			}
+		}
+		if !resp.Continue {
+			eventLog.Infof("interceptor stopped trigger processing: %v", resp.Status.Err())
+			return
+		}
+	}
+
+	trItems, err := r.selectTriggers(g.TriggerSelector.NamespaceSelector, g.TriggerSelector.LabelSelector)
+	if err != nil {
+		return
+	}
+
+	// Create a new HTTP request that contains the body and header from any interceptors in the TriggerGroup
+	// This request will be passed on to the triggers in this group
+	triggerReq := request.Clone(request.Context())
+	triggerReq.Header = header
+	triggerReq.Body = ioutil.NopCloser(bytes.NewBuffer(payload))
+
+	wg.Add(len(trItems))
+	for _, t := range trItems {
+		go func(t triggersv1.Trigger) {
+			defer wg.Done()
+			// TODO(dibyom): We might be able to get away with only cloning if necessary
+			// i.e. if there are interceptors and iff those interceptors will modify the body/header (i.e. webhook)
+			localRequest := triggerReq.Clone(triggerReq.Context())
+			r.processTrigger(t, el, localRequest, event, eventID, log, extensions)
+		}(*t)
+	}
+}
+
+func (r Sink) selectTriggers(namespaceSelector triggersv1.NamespaceSelector, labelSelector *metav1.LabelSelector) ([]*triggersv1.Trigger, error) {
+	var trItems []*triggersv1.Trigger
+	var err error
+	targetLabels := labels.Everything()
+	if labelSelector != nil {
+		targetLabels, err = metav1.LabelSelectorAsSelector(labelSelector)
+		if err != nil {
+			r.Logger.Errorf("failed to create label selector: %v", err)
+			return nil, err
+		}
+	}
+	var triggerFunc func() ([]*triggersv1.Trigger, error)
+	switch {
+	case len(namespaceSelector.MatchNames) == 1 && namespaceSelector.MatchNames[0] == "*":
+		triggerFunc = func() ([]*triggersv1.Trigger, error) {
+			return r.TriggerLister.List(targetLabels)
+		}
+	case len(namespaceSelector.MatchNames) != 0:
+		triggerFunc = func() ([]*triggersv1.Trigger, error) {
+			var trList []*triggersv1.Trigger
+			for _, v := range namespaceSelector.MatchNames {
+				trNsList, err := r.TriggerLister.Triggers(v).List(targetLabels)
+				if err != nil {
+					return nil, err
+				}
+				trList = append(trList, trNsList...)
+			}
+			return trList, nil
+		}
+	case len(namespaceSelector.MatchNames) == 0:
+		if labelSelector != nil {
+			triggerFunc = func() ([]*triggersv1.Trigger, error) {
+				return r.TriggerLister.Triggers(r.EventListenerNamespace).List(targetLabels)
+			}
+		}
+	}
+	if triggerFunc == nil {
+		return trItems, nil
+	}
+	trList, err := triggerFunc()
+	if err != nil {
+		r.Logger.Errorf("Error getting Triggers: %v", err)
+		return nil, err
+	}
+	trItems = append(trItems, trList...)
+
+	return trItems, nil
+}
+
+func (r Sink) processTrigger(t triggersv1.Trigger, el *triggersv1.EventListener, request *http.Request, event []byte, eventID string, eventLog *zap.SugaredLogger, extensions map[string]interface{}) {
+	log := eventLog.With(zap.String(triggers.TriggerLabelKey, t.Name))
+
+	finalPayload, header, iresp, err := r.ExecuteTriggerInterceptors(t, request, event, log, eventID, extensions)
+	if err != nil {
+		log.Error(err)
+		return
 	}
 
 	if iresp != nil {
 		if !iresp.Continue {
 			log.Infof("interceptor stopped trigger processing: %v", iresp.Status.Err())
-			return iresp.Status.Err()
+			return
 		}
 	}
 
@@ -259,16 +395,15 @@ func (r Sink) processTrigger(t triggersv1.Trigger, request *http.Request, event
 		r.TriggerTemplateLister.TriggerTemplates(t.Namespace).Get)
 	if err != nil {
 		log.Error(err)
-		return err
+		return
 	}
-	extensions := map[string]interface{}{}
 	if iresp != nil && iresp.Extensions != nil {
 		extensions = iresp.Extensions
 	}
 	params, err := template.ResolveParams(rt, finalPayload, header, extensions)
 	if err != nil {
 		log.Error(err)
-		return err
+		return
 	}
 
 	log.Infof("ResolvedParams : %+v", params)
@@ -276,16 +411,22 @@ func (r Sink) processTrigger(t triggersv1.Trigger, request *http.Request, event
 
 	if err := r.CreateResources(t.Namespace, t.Spec.ServiceAccountName, resources, t.Name, eventID, log); err != nil {
 		log.Error(err)
-		return err
+		return
 	}
 	go r.recordResourceCreation(resources)
-	return nil
+	r.emitEvents(r.EventRecorder, el, events.TriggerProcessingSuccessfulV1, nil)
+	r.sendCloudEvents(request.Header, *el, eventID, events.TriggerProcessingSuccessfulV1)
+
+}
+
+func (r Sink) ExecuteTriggerInterceptors(t triggersv1.Trigger, in *http.Request, event []byte, log *zap.SugaredLogger, eventID string, extensions map[string]interface{}) ([]byte, http.Header, *triggersv1.InterceptorResponse, error) {
+	return r.ExecuteInterceptors(t.Spec.Interceptors, in, event, log, eventID, fmt.Sprintf("namespaces/%s/triggers/%s", t.Namespace, t.Name), t.Namespace, extensions)
 }
 
 // ExecuteInterceptor executes all interceptors for the Trigger and returns back the body, header, and InterceptorResponse to use.
 // When TEP-0022 is fully implemented, this function will only return the InterceptorResponse and error.
-func (r Sink) ExecuteInterceptors(t triggersv1.Trigger, in *http.Request, event []byte, log *zap.SugaredLogger, eventID string) ([]byte, http.Header, *triggersv1.InterceptorResponse, error) {
-	if len(t.Spec.Interceptors) == 0 {
+func (r Sink) ExecuteInterceptors(trInt []*triggersv1.TriggerInterceptor, in *http.Request, event []byte, log *zap.SugaredLogger, eventID string, triggerID string, namespace string, extensions map[string]interface{}) ([]byte, http.Header, *triggersv1.InterceptorResponse, error) {
+	if len(trInt) == 0 {
 		return event, in.Header, nil, nil
 	}
 
@@ -294,16 +435,16 @@ func (r Sink) ExecuteInterceptors(t triggersv1.Trigger, in *http.Request, event
 	request := triggersv1.InterceptorRequest{
 		Body:       string(event),
 		Header:     in.Header.Clone(),
-		Extensions: map[string]interface{}{}, // Empty extensions for the first interceptor in chain
+		Extensions: extensions,
 		Context: &triggersv1.TriggerContext{
 			EventURL: in.URL.String(),
 			EventID:  eventID,
 			// t.Name might not be fully accurate until we get rid of triggers inlined within EventListener
-			TriggerID: fmt.Sprintf("namespaces/%s/triggers/%s", t.Namespace, t.Name), // TODO: t.Name might be wrong
+			TriggerID: triggerID,
 		},
 	}
 
-	for _, i := range t.Spec.Interceptors {
+	for _, i := range trInt {
 		if i.Webhook != nil { // Old style interceptor
 			body, err := extendBodyWithExtensions([]byte(request.Body), request.Extensions)
 			if err != nil {
@@ -315,7 +456,7 @@ func (r Sink) ExecuteInterceptors(t triggersv1.Trigger, in *http.Request, event
 				URL:    in.URL,
 				Body:   ioutil.NopCloser(bytes.NewBuffer(body)),
 			}
-			interceptor := webhook.NewInterceptor(i.Webhook, r.HTTPClient, t.Namespace, log)
+			interceptor := webhook.NewInterceptor(i.Webhook, r.HTTPClient, namespace, log)
 			res, err := interceptor.ExecuteTrigger(req)
 			if err != nil {
 				return nil, nil, nil, err
